@@ -30,6 +30,7 @@
   #:use-module (oop goops)
   #:use-module (system foreign)
   #:use-module (rnrs bytevectors)
+  #:use-module (ice-9 format)
   #:use-module (ice-9 match)
   #:use-module (ice-9 threads)
   #:use-module (ice-9 receive)
@@ -75,8 +76,8 @@
 	    n-chan->symbol
 	    im-collect
 
-	    im-display-channel
-	    im-display))
+	    im-display
+            im-display-channel))
 
 
 (g-export im-size
@@ -149,14 +150,17 @@
 	       (>= height 0)
 	       (integer? n-chan)
 	       (>= n-chan 1)
-	       (let ((n-cell (* width height)))
-		 (and (and-l (map f32vector? idata))
-		      (apply = (cons n-cell
-				     (map f32vector-length idata)))
-		      (apply <= (cons 0.0
-				      (sort (map f32vector-min idata) <)))
-		      (apply >= (cons 255.0
-				      (sort (map f32vector-max idata) >))))))))))
+               (= n-chan (length idata))
+               (and-l (map f32vector? idata)))))))
+
+#!
+(apply = (cons n-cell
+               (map f32vector-length idata)))
+(apply <= (cons 0.0
+                (sort (map f32vector-min idata) <)))
+(apply >= (cons 255.0
+                (sort (map f32vector-max idata) >)))
+!#
 
 (define (im-binary? image)
   (match image
@@ -166,10 +170,15 @@
 	    ((c)
 	     (receive (n-val vals)
 		 (f32vector-count-distinct c)
-	       (and (= n-val 2)
-		    (float-member 0.0 vals)
-		    (float-member 255.0 vals)))))
-	  #t))))
+               (match vals
+                 ((a) (or (float=? a 0.0 0)
+                          (float=? a 255.0 0)))
+                 ((a b) (or (and (float=? a 0.0 0)
+                                 (float=? b 255.0 0))
+                            (and (float=? a 255.0 0)
+                                 (float=? b 0.0 0))))
+                 (else
+                  #f)))))))))
 
 (define-method (im-gray? (image <list>))
   (match image
@@ -297,13 +306,10 @@
     ((k val)
      (match image
        ((width height n-chan idata)
-	(if (and (>= val 0.0)
-		 (<= val 255.0))
-	    (if (and (>= k 0)
-		     (< k n-chan))
-		(im-channel-set! (list-ref idata k) i j width height val)
-		(error "Out of bound: " k))
-	    (error "Invalid pixel value: " val)))))))
+        (if (and (>= k 0)
+                 (< k n-chan))
+            (im-channel-set! (list-ref idata k) i j width height val)
+            (error "Out of bound: " k)))))))
 
 (define (im-fast-set! image i j . rest)
   (match (match rest
@@ -361,29 +367,33 @@
   (string-append "Channel "
 		 (number->string (+ n 1))))
 
-(define* (im-display-channel channel width height
-			     #:key (proc identity)
-			     (port (current-output-port)))
-  (do ((i 0
-	    (+ i 1)))
-	((= i height))
-      (do ((j 0
-	      (+ j 1)))
-	  ((>= j width)
-	   (newline port))
-	(display "  ")
-	(display (proc (im-fast-channel-ref channel i j width)) port))))
-
 (define* (im-display image
-		     #:key (proc identity)
+		     #:key (proc #f)
 		     (port (current-output-port)))
   (match image
     ((width height n-chan idata)
      (for-each (lambda (channel i)
-		 (display (channel-name i) port)
-		 (newline port)
+                 (format port "\n~A\n\n" (channel-name i))
 		 (im-display-channel channel width height
-				     #:proc proc
-				     #:port port))
+                                     #:proc proc #:port port))
 	 idata
-       (iota n-chan)))))
+       (iota n-chan))))
+  (newline port))
+
+(define* (im-display-channel channel width height
+			     #:key (proc #f)
+			     (port (current-output-port)))
+  (let ((proc (if proc
+                  proc
+                  (lambda (val)
+                    (if (float>=? val 1000.0 0)
+                        (format #f "~9e" val)
+                        (format #f "~9,5,,,f" val))))))
+    (do ((i 0
+	    (+ i 1)))
+	((= i height))
+      (do ((j 0
+	      (+ j 1)))
+	  ((>= j width) (newline port))
+        (format port "  ~A"
+                (proc (im-fast-channel-ref channel i j width)))))))
