@@ -180,7 +180,20 @@
 	 (center (make-test-image-gray-type 'center))
 	 (square (make-test-image-gray-type 'square))
 	 (diamond (make-test-image-gray-type 'diamond))
+         (full (make-test-image-gray-type 'full))
 	 (img-1 (make-test-image-rgb 3 2)))
+    (assert-true (im-binary? empty))
+    (assert-true (im-binary? full))
+    (assert-true (im-binary? center))
+    (im-set! empty 0 0 -1.0)
+    (assert-false (im-binary? empty))
+    (im-set! empty 0 0 256.0)
+    (assert-false (im-binary? empty))
+    (im-set! empty 0 0 0.0)
+    (im-set! empty 0 1 1.0)
+    (im-set! empty 0 2 255.0)
+    (assert-false (im-binary? empty))
+    (assert-false (im-binary? img-1))
     (assert-true (im-image? empty))
     (assert-true (im-image? center))
     (assert-true (im-image? square))
@@ -188,28 +201,28 @@
     (assert-true (im-gray? empty))
     (assert-false (im-rgb? empty))
     (assert-true (im-rgb? img-1))
-    (assert-false (im-gray? img-1))
-    (im-fast-set! empty 0 0 -1.0)
-    (assert-false (im-image? empty))
-    (im-fast-set! empty 0 0 256.0)
-    (assert-false (im-image? empty))))
+    (assert-false (im-gray? img-1))))
 
 
 (define-method (test-impex (self <guile-cv-tests-cv>))
   (let* ((i-dir (getenv "TEST_IMAGES_PATH"))
 	 (t-file "/tmp/test.png")
 	 (b-file (string-append i-dir "/pp-17-label.png"))
-	 (l-file (string-append i-dir "/pp-17-bf.png")))
+	 (l-file (string-append i-dir "/pp-17-bf.png"))
+         (g-file (string-append i-dir "/gnu.png")))
     (assert-equal '(85 95 1) (im-size b-file))
     (assert ((@@ (cv impex) vigra-load-gray-image) b-file 85 95))
     (assert-equal '(85 95 3) (im-size l-file))
     (assert ((@@ (cv impex) vigra-load-rgb-image) l-file 85 95))
     (let ((bw (im-load b-file))
-	  (bf (im-load l-file)))
+	  (bf (im-load l-file))
+          (gnu (im-load g-file)))
       (assert-equal '(85 95 1) (im-size bw))
       (assert ((@@ (cv impex) vigra-save-gray-image) bw t-file 85 95))
       (assert-equal '(85 95 3) (im-size bf))
-      (assert ((@@ (cv impex) vigra-save-rgb-image) bf t-file 85 95)))))
+      (assert ((@@ (cv impex) vigra-save-rgb-image) bf t-file 85 95))
+      (assert-equal '(148 125 4) (im-size gnu))
+      (assert ((@@ (cv impex) vigra-save-rgba-image) gnu t-file 148 125)))))
 
 (define-method (test-im-range (self <guile-cv-tests-cv>))
   (let ((gray (make-test-image-gray 3 2))
@@ -286,9 +299,9 @@
     (assert-true (im-=? (im-transpose a) a'))
     (assert-true (im-=? (im-transpose a') a))))
 
-(define-method (test-im-inverse (self <guile-cv-tests-cv>))
+(define-method (test-im-invert (self <guile-cv-tests-cv>))
   (let* ((d `(2 3 1 (,#f32(2.0 4.0 8.0 16.0 32.0 64.0))))
-         (di (im-inverse d))
+         (di (im-invert d))
          (di-chan (im-channel di 0))
          (di-check '(1/2 1/4 1/8 1/16 1/32 1/64)))
     (do ((i 0
@@ -433,37 +446,37 @@
     (assert-true (im-=? (im-scrap a 9 #:pred =) c))
     (assert-true (im-=? (im-scrap a 9 #:pred >) d))))
 
-(define (make-features-images)
-  (let ((idx '((0 0)
-               (0 1) (0 2) (0 3)
-               (1 0) (2 0) (3 0)
-               (1 3) (2 3) (3 3)
-               (3 0) (3 1) (3 2)
-               (3 3)))
-        (img-1 (im-make 4 4 1 1.0))
-        (img-2 (im-make 4 4 3 1.0))
-        (img-3 (im-make 4 4 4 1.0)))
-    (for-each (lambda (pos)
-		(match pos
-		  ((i j)
-                   (im-set! img-1 i j 0.0))))
-	idx)
-    (for-each (lambda (pos)
-		(match pos
-		  ((i j)
-                   (im-set! img-2 i j 0 0.0)
-                   (im-set! img-2 i j 1 0.0)
-                   (im-set! img-2 i j 2 0.0))))
-	idx)
-    (values img-1 img-2 img-3)))
+(define-method (test-features-1 (self <guile-cv-tests-cv>))
+  (receive (trigo-x trigo-y)
+      ((@@ (cv features) eigen-vigra-coord->trigono-coord) 1 1)
+    (assert-numeric-= 135
+                      (radian->degree (atan trigo-y trigo-x))
+                      1.0e-4))
+  (receive (trigo-x trigo-y)
+      ((@@ (cv features) eigen-vigra-coord->trigono-coord) -1 1)
+    (assert-numeric-= 45
+                      (radian->degree (atan trigo-y trigo-x))
+                      1.0e-4))
+  (receive (trigo-x trigo-y)
+      ((@@ (cv features) eigen-vigra-coord->trigono-coord) -1 -1)
+    (assert-numeric-= -45
+                      (radian->degree (atan trigo-y trigo-x))
+                      1.0e-4))
+  (receive (trigo-x trigo-y)
+      ((@@ (cv features) eigen-vigra-coord->trigono-coord) 1 -1)
+    (assert-numeric-= -135
+                      (radian->degree (atan trigo-y trigo-x))
+                      1.0e-4)))
 
-(define-method (test-features (self <guile-cv-tests-cv>))
-  (receive (img-1 img-2 img-3)
-      (make-features-images)
-    (let ((l-img (im-label img-1)))
-      (assert-exception (im-features img-3 l-img))
-      (assert-exception (im-features img-1 img-2))
-      (assert (im-features img-1 l-img)))))
+(define-method (test-features-2 (self <guile-cv-tests-cv>))
+  (let* ((i-dir (getenv "TEST_IMAGES_PATH"))
+	 (i-file (string-append i-dir "/pp-17-bf.png"))
+         (a (im-load i-file))
+         (b (im-rgb->gray a))
+         (c (im-threshold b 136))
+         (d (im-label c)))
+    (assert (im-features a d))
+    (assert (im-features b d))))
 
 (define %texture-test-image
   (list 4 4 1
@@ -533,4 +546,10 @@
       (zip (im-reduce %glcm-test-result + 0)
            '(12.0 9.0 12.0 9.0))))
 
+(define-method (test-im-convolve (self <guile-cv-tests-cv>))
+  (let ((i1 %texture-test-image)
+        (i2 %glcm-test-result))
+    (assert-true (im-=? i1 (im-convolve i1 %k-identity)))
+    (assert-true (im-=? i2 (im-convolve i2 %k-identity)))))
+
 (exit-with-summary (run-all-defined-test-cases))
